=begin
**
** frame.rb
**
** Desc:
**  Small application to configure and launch the dradis client.
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

require 'fileutils'

module Core
  
  # The main window, it contains a tabbed control with tabs to launch and 
  # configure both the client and the server.
  class MiniFrame < Wx::MiniFrame
    #===================================================== private methods
    private 

    #----------------------------------------------------- client tab
    
    # The group of widgets that displays the current options for the client is
    # created in this method. This includes Server IP and TCP port, and the 
    # interval between refresh queries for the KnowledgeBase syncronisation.
    def setup_client_options_box(c_panel)
      c_opt_title = Wx::StaticBox.new(c_panel, Wx::ID_ANY, '')
      c_opt_box = Wx::StaticBoxSizer.new(c_opt_title, Wx::VERTICAL)

      # client controls
      @client_srv_ip = Wx::TextCtrl.new(c_panel, Wx::ID_ANY)
      @client_srv_port = Wx::TextCtrl.new(c_panel, Wx::ID_ANY)
      @client_username = Wx::TextCtrl.new(c_panel, Wx::ID_ANY)
      @client_password = Wx::TextCtrl.new(c_panel, Wx::ID_ANY, '',
          Wx::DEFAULT_POSITION, Wx::DEFAULT_SIZE, Wx::TE_PASSWORD)
      @client_refresh = Wx::SpinCtrl.new(c_panel, Wx::ID_ANY)
      @client_connect = Wx::Button.new(c_panel, Wx::ID_ANY, 'Connect')
      @exit_button = Wx::Button.new(c_panel, Wx::ID_ANY, 'Exit')

      # build the grid layout. each row consists of a label and a control
      c_opt_grid = Wx::FlexGridSizer.new(2)
      c_opt_grid.add_growable_col(1)
      c_opt_grid.add( Wx::StaticText.new(c_panel, Wx::ID_ANY, 'Server IP: '), 0, Wx::ALL|Wx::ALIGN_RIGHT, 5)
      c_opt_grid.add( @client_srv_ip, 0, Wx::ALL|Wx::ALIGN_LEFT|Wx::EXPAND, 2)
      c_opt_grid.add( Wx::StaticText.new(c_panel, Wx::ID_ANY, 'Server TCP Port: '), 0, Wx::ALL|Wx::ALIGN_RIGHT, 5)
      c_opt_grid.add( @client_srv_port, 0, Wx::ALL|Wx::ALIGN_LEFT|Wx::EXPAND, 2)
      c_opt_grid.add( Wx::StaticText.new(c_panel, Wx::ID_ANY, 'Username: '), 0, Wx::ALL|Wx::ALIGN_RIGHT, 5)
      c_opt_grid.add( @client_username, 0, Wx::ALL|Wx::ALIGN_LEFT|Wx::EXPAND, 2)
      c_opt_grid.add( Wx::StaticText.new(c_panel, Wx::ID_ANY, 'Password: '), 0, Wx::ALL|Wx::ALIGN_RIGHT, 5)
      c_opt_grid.add( @client_password, 0, Wx::ALL|Wx::ALIGN_LEFT|Wx::EXPAND, 2)
      c_opt_grid.add( Wx::StaticText.new(c_panel, Wx::ID_ANY, 'Refresh view every (in secs): '), 0, Wx::ALL|Wx::ALIGN_RIGHT, 5)
      c_opt_grid.add( @client_refresh, 0, Wx::ALL|Wx::ALIGN_LEFT, 2)
      c_opt_grid.layout()

      c_opt_box.add(c_opt_grid, 1, Wx::ALL|Wx::EXPAND)
      c_opt_box.add_spacer(5)
      c_opt_grid = Wx::FlexGridSizer.new(2)
      c_opt_grid.add_growable_col(1)
      c_opt_grid.add(@exit_button, 0, Wx::ALL|Wx::ALIGN_RIGHT, 5)
      c_opt_grid.add( @client_connect, 0, Wx::ALL|Wx::ALIGN_RIGHT, 5 )
      c_opt_box.add(c_opt_grid, 1, Wx::ALL|Wx::ALIGN_RIGHT)
      return c_opt_box
    end

    # Setup the widgets of the 'client tab'. This is made up of a panel, the 
    # 'launch' button and the options control box. For more info see 
    # setup_client_options_box().
    def setup_client_panel(notebook)
      # setup the panel and the main sizer
      c_panel = Wx::Panel.new(notebook)
      c_box = Wx::BoxSizer.new(Wx::VERTICAL)

      # the options box
      c_opt_box = setup_client_options_box(c_panel)
      c_box.add( c_opt_box, 1, Wx::ALL|Wx::EXPAND, 5)

      # launch the client button
      @btn_client = Wx::Button.new(c_panel, Wx::ID_ANY, 'launch')
      @btn_client.set_cursor( Wx::Cursor.new(Wx::CURSOR_HAND) )
      c_box.add( @btn_client, 0, Wx::ALL|Wx::EXPAND, 5 )

      c_panel.set_sizer_and_fit(c_box)
      return c_panel
    end

    #----------------------------------------------------- main window layout

    # Setup the layout of the main window. This includes the tabbed widget 
    # (Wx::AuiNotebook) and the 'exit' button.
    def setup_controls()
      vsizer = Wx::BoxSizer.new(Wx::VERTICAL)

      # setup notebook
      notebook = Wx::AuiNotebook.new(self, 
                                      Wx::ID_ANY, Wx::DEFAULT_POSITION, 
                                      Wx::DEFAULT_SIZE, Wx::AUI_NB_BOTTOM)

      # the client panel 
      c_panel = setup_client_panel(notebook)
      notebook.add_page( c_panel, 'client')

      notebook.show()


      vsizer.add(notebook,1, Wx::EXPAND)
      vsizer.add_spacer(10)
      vsizer.layout()

      self.set_sizer(vsizer)
    end

    def setup_events()
      # exit button
      evt_button(@exit_button) { exit }

      # ----- client tab
      # apply pressed
      evt_button(@client_connect) { on_btn_client_connect() }
    end

    def on_btn_client_connect()
      host  = @client_srv_ip.value
      port = @client_srv_port.value
      refresh = @client_refresh.value
      username = @client_username.value
      password = @client_password.value
      puts 'Saving configuration...'
      $config.put_option(:restful_server, "https://#{username}:#{password}@#{host}:#{port}/")
      $config.put_option(:ui_refresh, (refresh * 1000).to_s)
      Dir.chdir(@client_path) do
        $config.save
      end
      puts 'end.'
      @client_connect.disable()
      self.close()
    end
 
    #===================================================== public methods
    public
    def initialize(parent, title)
      super( parent, Wx::ID_ANY, Core::VERSION::string(), Wx::Point.new(200,  200),
              Wx::Size.new(450, 240), Wx::DEFAULT_FRAME_STYLE|Wx::STAY_ON_TOP)

      setup_controls()
      setup_events()
    end

    def fill_info( client_conf = {})
      # client: server ip, port and refresh interval
      @client_srv_ip.value = client_conf.fetch(:server_ip, '127.0.0.1')
      @client_srv_port.value = client_conf.fetch(:server_port, '3004')
      @client_username.value = client_conf.fetch(:username, '127.0.0.1')
      @client_password.value = client_conf.fetch(:password, '3004')
      @client_refresh.value = client_conf.fetch(:refresh, 3)
      @client_path = client_conf.fetch(:client_path, '.')
    end
  end
end
